
# This procedure average chemical shifts from peak lists and reports assignment errors
# Written by Pascal Mercier
# March 7, 2000
# the function argmuments are peak lists
# this routine needs the Hasting distribution of nmrview
#
#
# 1994
# Original "fillshift" program written by Leigh Willard &
# Stephane Gagne in the lab of Brian Sykes.  (non-GUI version)
#
#
# March 2003:
# Modified by Leigh Willard, labo de Stephane Gagne
# Universite Laval
# to speed up the procedure, which involved a complete re-write
# of the "extract_chemicalshifts" procedure.
#
#############################################################################



#........................................................................
# This is the subroutine re-written by Leigh.  It does the same
# thing as the original subroutine, only significantly faster
#
# For example, on a sequence of 112 residues, with 1799 atoms in total,
# and using 7 .xpk files, the entire program (not just this section)
# has been timed at (on my watch):
# OLD: 50 seconds
# NEW: 4 seconds
#
# Note that the results are saved in the same array as for the
# original subroutine, that is, in an array with the format:
#     array chemicalshift(i) { [PKLIST.PKNUM CHEMSHIFT]* } 
#     where i=the atom number in the sequence
#     an example is  chemicalshift(2) = { hsqc.2 118.5 tocsy.5 118.25 }
#

proc extract_chemicalshifts { allpeaklist } {
global chemicalshift

# 	foreach .xpk file
#	read the file, put info into chemical shift array
	foreach peaklist $allpeaklist {

#		find out what each of the dimensions is
		set dimlabels [nv_peak label $peaklist]

#		foreach peak/line in the peaklist file
#			foreach dimension, extract the info.
		set numpeaks [nv_peak n $peaklist]
		for { set i 0 } { $i < $numpeaks } { incr i } {
			foreach dim $dimlabels {

#				extract & store information
				set atom [nv_peak elem $dim.L $peaklist.$i]
				set atomnum [nv_atom num $atom]
				set cs [nv_peak elem $dim.P $peaklist.$i]
				append chemicalshift($atomnum) "$peaklist.$i $cs "
	
			}
		}
	}
}



#------------------------------------------------------------------------------
# sort the "hits" array for one atom, so that it is clearer
# to read on output.
proc sort_hits { hits } {

#	might as well go and put the first element in, then
#	set the counter i accordingly.
	append sorted "[lindex $hits 0] [lindex $hits 1]"
	set numbersort 2 

	set numberhits [llength $hits]
	for {set i 3} {$i<$numberhits} {incr i 2} {

		set done 0
		for {set j 1} {($j < $numbersort) && ($done == 0)} {incr j 2} {

			if { [lindex $hits $i] < [lindex $sorted $j] } {
				set sorted [linsert $sorted [expr $j-1] \
					[lindex $hits [expr $i-1]] \
					[lindex $hits $i]]
				set numbersort [expr $numbersort + 2]
				set done 1
			}
		}

#		we've made it to the end without finding anything bigger
		if { $done == 0 } {
			append sorted " "
			append sorted "[lindex $hits [expr $i-1]] [lindex $hits $i]"
			set numbersort [expr $numbersort + 2]
		}
	}

	return $sorted

}


#------------------------------------------------------------------------------
# check each atom to see if it has a violation.  
# there are 2 types of violations possible:
#	TYPE 1) deviations from the average value
#	TYPE 2) deviations from the ppm.out value
#
proc check_violations { i hits resname atomname seqnum average current minimum maximum} {
global nitrogentolerance carbontolerance protontolerance

# i = the index of the atom number for atomname, resname, and seqnum
# hits = array of matching chemical shifts in all of the experiments
# average = the already calculated average
# current = value from ppm.out file.


# 	get tolerance
	switch [string range [nv_atom elem aname $i] 0 0] {
		N {set tolerance $nitrogentolerance}
		C {set tolerance $carbontolerance}
		H {set tolerance $protontolerance}
		default puts "error:  can't get tolerance"
	}


#	quick check to see if there is a violation, by checking
#	the end points.  If there is no violation, then don't
#	bother with the rest of this procedure
	set check1 [expr [expr abs ($average - $minimum)] <= $tolerance]
	set check2 [expr [expr abs ($average - $maximum)] <= $tolerance]
#	there might not be a ppm value assigned.
	if {$current < -999} { 
		set check3 1
		set check4 1
	} else {
		set check3 [expr [expr abs ($current - $minimum)] <= $tolerance]
		set check4 [expr [expr abs ($current - $maximum)] <= $tolerance]
	}

	if { [expr $check1 && $check2 && $check3 && $check4 ] } { return }
	

#	sort the "hits" array to make nicer looking output
#	call it "sorted" now.  Use it from here on in.
	set sorted [sort_hits $hits]


	set numberhits [llength $sorted]
	for {set j 1} {$j<$numberhits} {incr j 2} {
		set ppmvalue [lindex $sorted $j]
		set peaklistvalue [lindex $sorted [expr $j - 1]]

#		VIOLATION TYPE 1
		set diff [expr abs($average - $ppmvalue)]
		if { $diff > $tolerance } {
			set viol1 [format "%9.3f" $diff]
		} else {
			set viol1 "         "
		}	
		

#		VIOLATION TYPE 2
		if {$current < -999} { 
			set viol2 "         "
		} else {
			set diff [expr abs ($current - $ppmvalue)]
			if { $diff > $tolerance } {
				set viol2 [format "%9.3f" $diff]
			} else {
				set viol2 "         "
			}
		}
		
# put out a line, even if there is no violation.
		set output [format "%-4s%6s%6s   %-15s%7.3f  %11s%11s" $resname $seqnum $atomname $peaklistvalue $ppmvalue $viol1 $viol2 ]
		.report.cpd24.01 insert end $output

	}

	.report.cpd24.01 insert end "------------------------------------------------------------------"
}
	



#........................................................................
proc fillshifts {} {

global Nv_Natoms

# first check if there is a sequence loaded in memory
if {$Nv_Natoms==0} {
	bell
	puts "Prior to using this program you need to have a sequence loaded"
	return
	}

if {![winfo exists .fillshifts]} {
       makefillshiftsavg
    } else {
     wm deiconify .fillshifts
    }
}



proc makefillshiftsavg {} {

set base .fillshifts

    toplevel $base -class Toplevel \
        -menu $base.m40 
    wm focusmodel $base passive
    wm geometry $base 409x431+130+177
    wm maxsize $base 1265 994
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base "Averaging of Chemical Shifts"


    label $base.lab19 \
        -borderwidth 0  -text {Available peak lists} -font {Helvetica -12}
    button $base.but21 \
         -activebackground #fe3640 -padx 11 -pady 4 -text "Add->" -font {Helvetica -12} -command {AddSelection}
    button $base.but22 \
         -activebackground #fe3640 -padx 11 -pady 4 -text "<-Remove" -font {Helvetica -12} -command {DeleteSelection} 
    label $base.lab27 \
        -borderwidth 1  -text {Peak lists used} -font {Helvetica -12}
    button $base.but28 \
        -activebackground #fe3640  -padx 11 -pady 4 -text Proceed -font {Helvetica -12 bold} -command {doshiftaveraging}
    button $base.but29 \
        -activebackground #fe3640  -padx 11 -pady 4 -text Quit -font {Helvetica -12 bold} -command {destroy .fillshifts}
    label $base.lab34 \
        -borderwidth 1  -font {Helvetica -10} -justify left \
        -text {Written by Pascal Mercier
March 7, 2000
University of Alberta
Brian Sykes lab
** Modified mars2003 by Leigh Willard, lab of Stephane Gagne, University Laval **} 


# TOLERANCES
    label $base.lab37 \
        -borderwidth 1  -text {Tolerance settings} -font {Helvetica -12 bold}

# H tolerance 
    frame $base.cpd39 \
        -borderwidth 1  -height 30 -relief raised -width 30 
    label $base.cpd39.01 \
        -anchor w  -relief groove -text H -font {Helvetica -12}
    entry $base.cpd39.02 \
         -highlightthickness 0 -font {Helvetica -12} -textvariable protontolerance \
         -width 0 

# what the heck is this stuff????
    frame $base.cpd39.fra41 \
        -borderwidth 1  -height 30 -relief raised -width 30 
    label $base.cpd39.fra41.01 \
        -anchor w  -relief groove -text H -font {Helvetica -12}
    entry $base.cpd39.fra41.02 \
         -highlightthickness 0  -width 0 
    frame $base.cpd39.fra42 \
        -borderwidth 1  -height 30 -relief raised -width 30 
    label $base.cpd39.fra42.01 \
        -anchor w  -relief groove -text H -font {Helvetica -12}
    entry $base.cpd39.fra42.02 \
         -highlightthickness 0  -width 0 
    frame $base.cpd39.fra42.03 \
        -borderwidth 1  -height 30 -relief raised -width 30 
    label $base.cpd39.fra42.03.04 \
        -anchor w  -relief groove -text H -font {Helvetica -12}
    frame $base.cpd39.cpd44 \
        -borderwidth 1 -height 30 -relief raised -width 30 
    label $base.cpd39.cpd44.01 \
        -anchor w -relief groove -text label -font {Helvetica -12}
    entry $base.cpd39.cpd44.02 \
        -cursor {} -highlightthickness 0  

# ???
    menu $base.m40 \
        -cursor {} 

# N tolerance
    frame $base.cpd43 \
        -borderwidth 1  -height 30 -relief raised -width 30 
    label $base.cpd43.01 \
        -anchor w  -relief groove -text N -font {Helvetica -12}
    entry $base.cpd43.02 \
         -highlightthickness 0 -textvariable nitrogentolerance -font {Helvetica -12}\
         
# C tolerance
    frame $base.cpd46 \
        -borderwidth 1  -height 30 -relief raised -width 30 
    label $base.cpd46.01 \
        -anchor w  -relief groove -text C -font {Helvetica -12}
    entry $base.cpd46.02 \
        -cursor {} -highlightthickness 0 -textvariable carbontolerance -font {Helvetica -12}\
         
# available peak lists frame?
    frame $base.cpd47 \
        -borderwidth 1  -height 30 -relief raised -width 30 
    listbox $base.cpd47.01 \
         \
        -font {Helvetica -12} \
        -xscrollcommand {.fillshifts.cpd47.02 set} \
        -yscrollcommand {.fillshifts.cpd47.03 set} 
    scrollbar $base.cpd47.02 \
        -borderwidth 1 -command {.fillshifts.cpd47.01 xview} -orient horizontal \
        -width 10 
    scrollbar $base.cpd47.03 \
        -borderwidth 1 -command {.fillshifts.cpd47.01 yview}  \
        -width 10 

# Peak List Used frame?
    frame $base.cpd48 \
        -borderwidth 1  -height 30 -relief raised -width 30 
    listbox $base.cpd48.01 \
        -font {Helvetica -12} \
        -xscrollcommand {.fillshifts.cpd48.02 set} \
        -yscrollcommand {.fillshifts.cpd48.03 set} 
    scrollbar $base.cpd48.02 \
        -borderwidth 1 -command {.fillshifts.cpd48.01 xview} -orient horizontal \
        -width 10 
    scrollbar $base.cpd48.03 \
        -borderwidth 1 -command {.fillshifts.cpd48.01 yview} -width 10 
    ###################
    # SETTING GEOMETRY
    ###################
    place $base.lab19 \
        -x 15 -y 5 -anchor nw -bordermode ignore 
    place $base.but21 \
        -x 162 -y 55 -width 80 -height 28 -anchor nw -bordermode ignore 
    place $base.but22 \
        -x 162 -y 95 -width 80 -height 28 -anchor nw -bordermode ignore 
    place $base.lab27 \
        -x 251 -y 5 -width 114 -height 18 -anchor nw -bordermode ignore 
    place $base.but28 \
        -x 5 -y 330 -width 131 -height 28 -anchor nw -bordermode ignore 
    place $base.but29 \
        -x 145 -y 330 -width 131 -height 28 -anchor nw -bordermode ignore 
    place $base.lab34 \
        -x 10 -y 370 -anchor nw -bordermode ignore 
    place $base.lab37 \
        -x 11 -y 185 -width 107 -height 18 -anchor nw -bordermode ignore 
    place $base.cpd39 \
        -x 10 -y 210 -width 65 -height 25 -anchor nw -bordermode ignore 
    pack $base.cpd39.01 \
        -in $base.cpd39 -anchor center -expand 0 -fill none -padx 2 -pady 2 \
        -side left 
    pack $base.cpd39.02 \
        -in $base.cpd39 -anchor center -expand 1 -fill x -padx 2 -pady 2 \
        -side right 
    place $base.cpd39.fra41 \
        -x 10 -y 209 -width 110 -height 25 -anchor nw -bordermode ignore 
    pack $base.cpd39.fra41.01 \
        -in $base.cpd39.fra41 -anchor center -expand 0 -fill none -padx 2 \
        -pady 2 -side left 
    pack $base.cpd39.fra41.02 \
        -in $base.cpd39.fra41 -anchor center -expand 1 -fill x -padx 2 \
        -pady 2 -side right 
    place $base.cpd39.fra42 \
        -x 10 -y 210 -width 110 -height 25 -anchor nw -bordermode ignore 
    pack $base.cpd39.fra42.01 \
        -in $base.cpd39.fra42 -anchor center -expand 0 -fill none -padx 2 \
        -pady 2 -side left 
    pack $base.cpd39.fra42.02 \
        -in $base.cpd39.fra42 -anchor center -expand 1 -fill x -padx 2 \
        -pady 2 -side right 
    place $base.cpd39.fra42.03 \
        -x 10 -y 209 -width 110 -height 25 -anchor nw -bordermode ignore 
    place $base.cpd39.cpd44 \
        -x 145 -y 50 -width 245 -height 25 -anchor nw -bordermode ignore 
    pack $base.cpd39.cpd44.01 \
        -in $base.cpd39.cpd44 -anchor center -expand 0 -fill none -padx 2 \
        -pady 2 -side left 
    pack $base.cpd39.cpd44.02 \
        -in $base.cpd39.cpd44 -anchor center -expand 1 -fill x -padx 2 \
        -pady 2 -side right 
    place $base.cpd43 \
        -x 10 -y 240 -width 65 -height 25 -anchor nw -bordermode ignore 
    pack $base.cpd43.01 \
        -in $base.cpd43 -anchor center -expand 0 -fill none -padx 2 -pady 2 \
        -side left 
    pack $base.cpd43.02 \
        -in $base.cpd43 -anchor center -expand 1 -fill x -padx 2 -pady 2 \
        -side right 
    place $base.cpd46 \
        -x 10 -y 270 -width 65 -height 30 -anchor nw -bordermode ignore 
    pack $base.cpd46.01 \
        -in $base.cpd46 -anchor center -expand 0 -fill none -padx 2 -pady 2 \
        -side left 
    pack $base.cpd46.02 \
        -in $base.cpd46 -anchor center -expand 1 -fill x -padx 2 -pady 2 \
        -side right 
    place $base.cpd47 \
        -x 10 -y 30 -width 145 -height 145 -anchor nw -bordermode ignore 
    grid columnconf $base.cpd47 0 -weight 1
    grid rowconf $base.cpd47 0 -weight 1
    grid $base.cpd47.01 \
        -in $base.cpd47 -column 0 -row 0 -columnspan 1 -rowspan 1 \
        -sticky nesw 
    grid $base.cpd47.02 \
        -in $base.cpd47 -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky ew 
    grid $base.cpd47.03 \
        -in $base.cpd47 -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky ns 
    place $base.cpd48 \
        -x 250 -y 30 -width 145 -height 145 -anchor nw -bordermode ignore 
    grid columnconf $base.cpd48 0 -weight 1
    grid rowconf $base.cpd48 0 -weight 1
    grid $base.cpd48.01 \
        -in $base.cpd48 -column 0 -row 0 -columnspan 1 -rowspan 1 \
        -sticky nesw 
    grid $base.cpd48.02 \
        -in $base.cpd48 -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky ew 
    grid $base.cpd48.03 \
        -in $base.cpd48 -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky ns 

bind .fillshifts.cpd47.01 <Double-1> {
	AddSelection
}

bind .fillshifts.cpd48.01 <Double-1> {
	DeleteSelection
}

# Clear each lists
.fillshifts.cpd47.01 delete 0 end
.fillshifts.cpd48.01 delete 0 end

#insert the list of available peak lists in the first list
foreach list [nv_peak list] { .fillshifts.cpd47.01 insert end $list}
}

#set default tolerance value from smg
set protontolerance 0.02
set nitrogentolerance 0.1
set carbontolerance 0.1

proc AddSelection {} {
	if {[selection own]==".fillshifts.cpd47.01"} {
   		.fillshifts.cpd48.01 insert end [selection get]
   		.fillshifts.cpd47.01 delete active
	}
}

proc DeleteSelection {} {
	if {[selection own]==".fillshifts.cpd48.01"} {
   		.fillshifts.cpd47.01 insert end [selection get]
   		.fillshifts.cpd48.01 delete active
	}
}





########################################
#
# This is where the fun begins
#
########################################

proc doshiftaveraging {} {

global nitrogentolerance carbontolerance protontolerance

# check if the user has chosen at least one peak list to work with
if {[.fillshifts.cpd48.01 size]==0} {
	bell
	puts "Please select at least one peak list"
	bell
return
}

# check if the user has tolerance values for H, N and C
set protontolerance    [.fillshifts.cpd39.02 get]
set nitrogrentolerance [.fillshifts.cpd43.02 get]
set carbontolerance    [.fillshifts.cpd46.02 get]

if {$protontolerance<0 || $nitrogrentolerance<0 || $carbontolerance<0} {
	bell
	puts "For the program to work you need to set tolerance values for proton, nitrogen and carbon atoms.  One of them is not set, or one of them is set to a negative value"
	bell
	return
}

###########################################
# core of the procedure
###########################################

global amino
global chemicalshift
global Nv_Natoms
set numberatoms $Nv_Natoms

# store all the peak lists chosen in a variable called allpeaklist
set allpeaklist [.fillshifts.cpd48.01 get 0 end]

puts "getting mean chemical shifts from BMRB"
  if {![info exists amino]} then {
	# the GetAminoTables is from the Hasting dribution of nmrview (script resdata.tcl)
        GetAminoTables
    }

puts "starting anaylsis... please be patient"; puts ""

# if the report window does not exist, make it, and if it does exists, clear the the list boxes

if {![winfo exists .report]} {
       makereportfillshifts
    } else {
     .report.cpd24.01 delete 0 end
     .report.cpd28.01 delete 0 end
}


#.......................................
# section that Leigh re-wrote
# initialize chemicalshifts 
for {set i 0} {$i<=$numberatoms} {incr i 1} {
	set chemicalshift($i) ""
}
extract_chemicalshifts $allpeaklist
# end section
#.......................................



# Average the shifts and report possible assignment errors


for {set i 0} {$i<=$numberatoms} {incr i 1} { 

  set averageresults($i) ""
  set hits $chemicalshift($i)
  set numberhits [llength $hits] 
  if {$hits!=""} {
	set resname         [nv_atom elem rname $i]
	set atomname        [nv_atom elem aname $i]
	set seqnum          [nv_atom elem seq $i]
	set minimum [lindex $hits 1]
	set maximum [lindex $hits 1]
	set total 0
	set numbers ""
	
	
	# get total chemical shifts, and counts.
	for {set j 1} {$j<$numberhits} {incr j 2} {
		set ppmvalue [lindex $hits $j]
		set total [expr $total + $ppmvalue]
		set minimum [expr {($ppmvalue < $minimum) ? $ppmvalue:$minimum}]
		set maximum [expr {($ppmvalue > $maximum) ? $ppmvalue:$maximum}]
	}

	set average [expr $total/$numberhits*2]
	set currentppm [nv_atom elem ppm $i]
	set BMRB $amino($resname,$atomname,shift)
	set deltaBMRB [expr $average - $BMRB]

	if {$currentppm>-1000} {
		set delta [expr $average-$currentppm]
		set averageresults($i) [format "%-4s%6s%6s%11.3f%11.3f%11.3f%11.3f" $resname $seqnum $atomname $average $delta $BMRB $deltaBMRB]
	} else {
		set delta ND
		set averageresults($i) [format "%-4s%6s%6s%11.3f%11s%11.3f%11.3f" $resname $seqnum $atomname $average $delta $BMRB $deltaBMRB]
	}

	
	check_violations $i $hits $resname $atomname $seqnum $average $currentppm $minimum $maximum

  }


}



for {set i 0} {$i<=$numberatoms} {incr i 1} { 
 if {$averageresults($i) !=""} {
	.report.cpd28.01 insert end $averageresults($i)
	}
}


#bell
puts ""; puts Done
# end of procedure
}




###########################
#
# Assign Shift procedure
#
###########################

proc AssignShift {} {
if {[selection own]==".report.cpd28.01"} {
	set selected [selection get]
	set residuenum [lindex $selected 1]
	set atomname [lindex $selected 2]
	set ppm [lindex $selected 3]
	set atomnumber [nv_atom num $residuenum.$atomname]
	puts "assign $residuenum.$atomname $ppm"
	nv_atom elem ppm $atomnumber $ppm
}
}


###########################
#
# Assign All Shift procedure
#
###########################

proc AssignAllShift {} {
for {set i 0} {$i<[.report.cpd28.01 size]} {incr i} {
	set currentline [.report.cpd28.01 get $i $i]
	set selected [lindex $currentline 0]
	set residuenum [lindex $selected 1]
	set atomname [lindex $selected 2]
	set ppm [lindex $selected 3]
	set atomnumber [nv_atom num $residuenum.$atomname]
	puts "assign $residuenum.$atomname $ppm"
	nv_atom elem ppm $atomnumber $ppm
}
}

###########################
#
# Save Shift report
#
###########################


proc SaveShiftReport {} {
global tk_version tk_strictMotif
set tk_strictMotif 0

set file [tk_getSaveFile -initialdir [pwd]  -initialfile "avg-shift.report"]

set createreportfile [open $file w]

puts $createreportfile "Peak lists used:"
puts $createreportfile [.fillshifts.cpd48.01 get 0 end]

puts $createreportfile "\n\nChemical Shift Violations:"
puts $createreportfile [format "%-4s%6s%6s   %-15s%7s  %11s%11s" RES NUM ATOM PEAKLIST PPM VIOL(xpk) VIOL(ppm)]

#write errors at the top
for {set i 0} {$i<[.report.cpd24.01 size]} {incr i} {
	set currentline [.report.cpd24.01 get $i $i]
	set currentline [lindex $currentline 0]
	puts $createreportfile $currentline
}

puts $createreportfile "\n\n\n"

puts $createreportfile [format "%-4s%6s%6s%11s%11s%11s%11s" RES NUM ATOM average delta BMRB deltaBMRB]
for {set i 0} {$i<[.report.cpd28.01 size]} {incr i} {
	set currentline [.report.cpd28.01 get $i $i]
	set currentline [lindex $currentline 0]
	puts $createreportfile $currentline
}


close $createreportfile
}



proc reportshiftsinterface {} {

if {![winfo exists .reportfillshifts]} {
       makereportfillshifts
    } else {
     wm deiconify .makereportfillshifts
    }
}

proc makereportfillshifts {} {

set base .report

    toplevel $base -class Toplevel \
         
    wm focusmodel $base passive
#    wm geometry $base 506x542+145+224
    wm geometry $base 506x647+145+224
    wm maxsize $base 1265 994
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base "Report"

#
#   violations table
#
    label $base.lab21 \
        -borderwidth 1  \
        -text {Chemical Shift Violations} -font {Helvetica -12 bold}
    label $base.lab22 \
        -borderwidth 1  -relief flat  -text [format "%-4s%6s%6s   %-15s%7s  %11s%11s" RES NUM ATOM PEAKLIST PPM VIOL(xpk) VIOL(ppm)] -font {courier -12} -justify left
    frame $base.cpd24 \
        -borderwidth 1  -height 30 -relief raised -width 30 
    listbox $base.cpd24.01 \
        -font {Courier -12} \
         -xscrollcommand {.report.cpd24.02 set} \
        -yscrollcommand {.report.cpd24.03 set} 
    scrollbar $base.cpd24.02 \
        -borderwidth 1 -command {.report.cpd24.01 xview} -orient horizontal \
        -width 10 
    scrollbar $base.cpd24.03 \
        -borderwidth 1 -command {.report.cpd24.01 yview} -width 10 

#
# chemical shifts averaging table
#
    label $base.lab25 \
        -borderwidth 1 -text {Chemical shifts averaging results} -font {Helvetica -12 bold}
    frame $base.cpd28 \
        -borderwidth 1  -height 30 -relief raised -width 30 
    listbox $base.cpd28.01 \
         \
         -font {Courier -12}\
         -xscrollcommand {.report.cpd28.02 set} \
        -yscrollcommand {.report.cpd28.03 set} 
    scrollbar $base.cpd28.02 \
        -borderwidth 1 -command {.report.cpd28.01 xview} -orient horizontal \
        -width 10 
    scrollbar $base.cpd28.03 \
        -borderwidth 1 -command {.report.cpd28.01 yview} -width 10 
    button $base.but29 \
         -padx 11 -pady 4 -text Assign -font {Helvetica -12 bold} -activebackground #fe3640 -command {AssignShift}
    button $base.but30 \
         -padx 11 -pady 4 -text {Assign All} -font {Helvetica -12 bold} -activebackground #fe3640 -command {AssignAllShift}
    button $base.but31 \
         -padx 11 -pady 4 -text {Save report} -font {Helvetica -12 bold} -activebackground #fe3640  -command {SaveShiftReport}
    button $base.but32 \
         -padx 11 -pady 4 -text Quit -font {Helvetica -12 bold} -activebackground #fe3640 -command {destroy .report} 
    label $base.lab34 \
        -borderwidth 1  -relief flat  -text [format "%-4s%6s%6s%11s%11s%11s%11s" RES NUM ATOM average delta BMRB deltaBMRB] -font {courier -12} -justify left
    ###################
    # SETTING GEOMETRY
    ###################
    place $base.lab21 \
        -x 10 -y 5 -anchor nw -bordermode ignore 
    place $base.lab22 \
        -x 15 -y 25 -anchor nw -bordermode ignore 
    place $base.cpd24 \
        -x 10 -y 40 -width 490 -height 250 -anchor nw -bordermode ignore 
    grid columnconf $base.cpd24 0 -weight 1
    grid rowconf $base.cpd24 0 -weight 1
    grid $base.cpd24.01 \
        -in $base.cpd24 -column 0 -row 0 -columnspan 1 -rowspan 1 \
        -sticky nesw 
    grid $base.cpd24.02 \
        -in $base.cpd24 -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky ew 
    grid $base.cpd24.03 \
        -in $base.cpd24 -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky ns 
    place $base.lab25 \
        -x 10 -y 310 -anchor nw -bordermode ignore 
    place $base.cpd28 \
        -x 15 -y 350 -width 485 -height 250 -anchor nw -bordermode ignore 
    grid columnconf $base.cpd28 0 -weight 1
    grid rowconf $base.cpd28 0 -weight 1
    grid $base.cpd28.01 \
        -in $base.cpd28 -column 0 -row 0 -columnspan 1 -rowspan 1 \
        -sticky nesw 
    grid $base.cpd28.02 \
        -in $base.cpd28 -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky ew 
    grid $base.cpd28.03 \
        -in $base.cpd28 -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky ns 
    place $base.but29 \
        -x 20 -y 605  -anchor nw -bordermode ignore 
    place $base.but30 \
        -x 95 -y 605 -anchor nw -bordermode ignore 
    place $base.but31 \
        -x 185 -y 605 -anchor nw -bordermode ignore 
    place $base.but32 \
        -x 290 -y 605  -anchor nw -bordermode ignore
    place $base.lab34 \
        -x 18 -y 333 -anchor nw -bordermode ignore 
}
