############################################################################
# Fillshift3 - calculate average chemical shifts and report
#              assignment errors.
#
# A program developed at Laval University and the University of Alberta.
#
# email to:	leigh@rsvs.ulaval.ca
# website:	http://nmr.ulaval.ca/labo/software.html
# Development:  Stephane Gagne
# Programmers:  Leigh Willard and Pascal Mercier
#
############################################################################
#
# To invoke:
#	type fs3 from with nmrview
#
# To install globally (for all users):
#	copy over this tcl program into
#	/usr/local/packages/nmrview5.0.4/nvtcl (or wherever your 
#		installation of nmrview is)
#
# 	make sure that the tclIndex file is writable by everyone,
#	OR make sure the file is writable for yourself, and start up
#	nmrview.  Everytime nmrview is run, it updates this file
#	based on an inventory of the nvtcl directory, and it needs
#	to be able to index the new program.
#
############################################################################
#
# Development History:
#
# Leigh Willard programmed version 3.X of this program, March 2003
#   this was a major re-write, resulting in 80% - 90% of the 
#   code being re-written.
# Pascal Mercier wrote version 2.X of this program, March 2000
# Leigh Willard wrote version 1.X  in 1994 (non-GUI version)
#
# --> v 3.6   bug fixes
# --> v 3.5   added enhancements:
# -->   ability to sort on any output column
# -->   ability to re-size windows
# -->   completely re-wrote the main GUI.
# -->   added an option for overlapping peaks
# --> v 3.4   changes and bug fixes suggested by Pascal Mercier
# -->   are incorported:
# -->   account for gaps (holes) in the numbering of peaklists
# -->   check for empty data "ND" before doing an assign all.
# -->   error output window - allow user to sort on different fields.
# -->   add option to include overlapping peaks in calculations
# --> v 3.3   the only change this version is to re-name 
# -->   "fillshiftG" to fs3, and call the name of the procedure "fs3"
# -->   instead of fillshift, and fix some help page text.
#
############################################################################
set VERSION "3.6, dec2003"
############################################################################
# END OF COMMENTS.  PROGRAM FOLLOWS.
############################################################################



#-------------------------------------------------------------------------
# [Leigh] It is important to tell the users what parts of the program the
# smg lab modified. 

proc give_smg_details {} {
	set f .details
	destroy $f
	toplevel $f

	wm title $f "MODIFICATION DETAILS"

	pack [text $f.tx -height 52 -width 60]
	pack [button $f.b1 -text Done -default active \
		-activebackground red -command {destroy .details; return}]

	$f.tx insert end {
CREDITS---------------------------------------------
Versions 3.*: Leigh Willard (programmer) & Stephane Gagne 
              Laval University
Versions 2.*: Pascal Mercier (programmer) & Brian Sykes 
              University of Alberta
Versions 1.*: Leigh Willard & Stephane Gagne

HISTORY---------------------------------------------
version 1.*: written by Willard & Gagne.  Non-gui version.  1994.
version 2.*: named "fillshift", a gui was added, major re-write.  
             written by Mercier & Sykes.  march 2002.
version 3.*: renamed "fs3" (to try to avoid confusion),
             major re-writes, speedups, add-ons.
             written by Willard & Gagne.  nov 2003.

ENHANCEMENTS WITH VERSION 3.X ----------------------
1) speed up the calculations significantly.
   For example, on a sequence of 112 residues, 
   with 1799 atoms in total, and using 7 .xpk files, 
   when the user clicks on "proceed", until the time that 
   the results are shown, the times are (timed on my watch):
       OLD: 50 seconds
       NEW: 4 seconds

2) completely re-write the violations section.
   We were not in agreement with how violations were
   computed before, so we have changed this section to
   have 2 types of violations:
   viol(avg) - those atoms which differ from the average
   viol(ppm) - those atoms which differ from the current ppm value

3) re-wrote the main GUI (window), and added these options:
   - Violations Ouput (show violations only, viol(avg), viol(ppm)
   - Options (C peak unfolding, overlapping peaks)
   - added Help and Credits pages

4) re-worked the reporting window to include:
   - ability to sort on output columns
   - ability to click on  lines
   - added help pages
   - added "ppm.out" column
   - in the "chemical shift averaging results", write a line for every
   residue/atom, instead of just putting a line whenever user had
   average data.  
   - re-did the way that violations are reported.

5) basically, I have re-written about 80%-90% of the code from
   version 2.X, but the program still maintains the "essence"
   of what it was before. 
----------------------------------------------------

}

#}


#-------------------------------------------------------------------------
# [Leigh] To give users a description of the options on the main panel. 

proc write_main_help {} {
	set f .help
	destroy $f
	toplevel $f

	wm title $f "FS3 HELP"

	pack [frame $f.fr]
	pack [text $f.fr.tx -height 40 -width 60 \
		-yscrollcommand {.help.fr.sb set} ] -side left
	pack [scrollbar $f.fr.sb -background lightblue \
		-activebackground lightblue \
        	-width 10 -borderwidth 1 -command {.help.fr.tx yview}  \
		 ] -fill y -side right
	pack [button $f.b1 -text Done -default active -activebackground red \
		-command {destroy .help; return}] -side bottom

	$f.fr.tx insert end {
This help describes the three categories of settings:
   Tolerance Settings
   Violation Output
   Options
All other functions on the main panel should be self-evident.
Note: there is a different Help page accessible from the report window.

----------------------------------------------------
Tolerance Settings
     the tolerance settings determine what will be flagged as a 
     violations.  We have added our own default values.
     
     Two types of violations are flagged:  those peaks which
     deviate from the ppm.out value, and those peaks which deviate
     from the average value.  More details are available from
     the Help page in the report window.

----------------------------------------------------
Violation Output:  
     This allows one to choose which columns/rows will 
     appear in the violations report.

     "show violations only" - normally if there is any
     violation for a given residue/atom, then a line is
     written out for EACH xpk file into the violations panel.
     If the user just wants to see the xpk file that contained
     the violation, then select this option.

     "viol(avg)" - there are two types of violations, which
     are described in detail in the Help section of the violations panel.
     Selecting this option will check for the first type
     of violation, which is a deviation from the average.

     "viol(ppm)" - there are two types of violations, which
     are described in detail in the Help section of the violations panel.
     Selecting this option will check for the second type
     of violation, which is a deviation from the ppm.out value.

----------------------------------------------------
Options:

     Automatic C peak unfolding:
        This may be useful in some cases, if the user thinks
        that the C peaks are folded.  The program attempts to
        unfold them, and will write a message to the nmrview main
        window for every peak that was unfolded.

        Unfolding works like this for any C atom:
           swsf = sweep width / spectrometer frequency
        take the expected value from the BMRB database and
        compare it against these 3 values:
           the current peak
           the current peak + swsf
           the current peak - swsf
        whichever value is closest to the expected value we will
        assume is the corrent current peak, and we will use that
        value from now on, after informing the user.
     
      Include Overlapping Peaks:
        Normally we ignore overlapping peaks (those lines from the 
        input files are discarded).  If, however, you prefer to have
        those lines read, then check this box.  The program will
        treat those lines as multiple data elements.
       
----------------------------------------------------
}
}

#-------------------------------------------------------------------------
# [Leigh] Give help on the "Report" screen, so users know exactly
# what each of the columns is. 

proc write_report_help {} {
	set f .help
	destroy $f
	toplevel $f

	wm title $f "REPORT HELP"

	pack [text $f.tx -height 50 -width 60]
	pack [button $f.b1 -text Done -default active \
		-activebackground red \
		-command {destroy .help; return}]

	$f.tx insert end {
------------------------------
--Chemical Shift Violations--
------------------------------
There are 2 types of chemical shift violations:

1) Vavg which are deviations from the average value
   For example: 
       - the average of all my .xpk files for ASP 3 N is 119.129, 
       - but in my hncaco.xpk file, the ppm for ASP 3 N is 118.994
       - my N tolerance is set to 0.1 (in the main panel)
       - 118.994 - 119.129 = -0.135, which is the VIOL(avg) value
       - the absolute value is 0.135, which is GREATER THAN
         the tolerance, so this is flagged as a violation.


2) Vppm which are deviations from the ppm.out value
   For example:
       - my ppm.out value for ASP 3 N is 119.118
       - but in my hncaco.xpk file, the ppm for ASP 3 N is 118.994
       - my N tolerance is set to 0.1 (in the main panel)
       - 118.994 - 119.118 = -0.124, which is the VIOL(ppm) value
       - the absolute value is 0.124, which is GREATER THAN
         the tolerance, so this is flagged as a violation.
------------------------------------
--Chemical Shift Averaging Results--
------------------------------------
RES =       3 letter residue name
NUM =       residue number
ATOM =      atom name
AVERAGE =   the average of all of the .xpk files for this residue/atom
PPM.OUT =   the current assigned value
deltaPPM =  (average - ppm.out) 
BMRB =      the corresponding residue/atom in the BMRB database
deltaBMRB = (average - BMRB)
------------------------------------
--ASSIGN--
------------------------------------
When the "Assign" button is pressed, the average from the
  currently highlighted entry from the "Chemical shift averaging results"i
  will be written out to the nmrview program.
When the "Assign All" button is pressed, 2 things happen:
  1) a copy of your current ppm.out values (from nrmview) is
     written out to a file, which is named something like 
     "ppm.out_27Nov03_10:35" 
  2) all of the "average" values will be written out to the
     nmrview program.
   NOTE:  your ppm.out file is not touched by fs3, but the
     values are sent to nmrview.
-------------------------------------
}
}

#--------------------------------------------------------------------------
# [Leigh] This procedure sets the title that will appear in
# the violations window, and in the report.  It changes depending
# on which calculations the user wants.

proc set_viol_title { base } {
global viol_title
global vb0 vb1 vb2

	set viol_title "RES  NUM ATOM PEAKLIST                                PPM"
	pack [frame .report.sort] -anchor w

	pack [label .report.sort.l1 -relief flat -text "     " \
		-font { courier -12} ] -side left
	pack [radiobutton .report.sort.b1 -padx 0 -pady 0 \
		-variable sortby -value 0 \
		-justify left  -selectcolor darkblue \
		-command {sort_viol 1 } ] -anchor w -side left
	pack [label .report.sort.l2 -relief flat \
		-text "                                                    " \
		-font { courier -12} ] -side left
	if { $vb1 } { 
		set viol_title [format "%s %s" $viol_title "   Vavg"]
		pack [radiobutton .report.sort.b2 -padx 0 -pady 0 \
			-variable sortby -value 1 \
			-selectcolor darkblue \
			-command {sort_viol 5 } ] -side left
		pack [label .report.sort.l4 -relief flat -text "   "\
			-font { courier -12} ] -side left
	}
	if { $vb2 } { 
		set viol_title [format "%s %s" $viol_title "   Vppm"]
		pack [radiobutton .report.sort.b3 -padx 0 -pady 0 \
			-variable sortby -value 2 \
			-selectcolor darkblue \
			-command {sort_viol 6 } ] -side left
	}

	pack [frame .report.title] -anchor w 
	pack [label .report.title.lab -font {courier -12} \
		-relief flat -text $viol_title ] -anchor w -side left
	
	.report.sort.b1 select


}

#--------------------------------------------------------------------------
# [Leigh] This procedure allows the user to click on one of the "report"
# listboxes (either "chemical shift violations", or "chemical shift
# averaging results") and have the appropriate atom in the other
# window highlighted.
#
# listone listtwo are the current and target listbox
#

proc select_violation {listone listtwo } {
global selectedlist

	set index [$listone curselection]
	if { $index == "" } {return;}

	set value [$listone get $index]

#	de-selection manually any previously  selected entries.
	set oldlist [lindex $selectedlist 0]
	set oldindex [lrange $selectedlist 1 end]
	foreach idx $oldindex {
		$oldlist itemconfigure $idx -bg {}
		$oldlist itemconfigure $idx -fg {}
	}

# they might have clicked on a line like --------------------------
# if we don't get what we expected, return without doing anything
	set ret [scan $value "%s %d %s" res num atom]
	if {$ret != 3} {
		return
	} 
	

# got to do this myself... no build in command to scan all
# of the lines in the listbox for a value.

#	for each line in the "chemical shifts averaging results"

#	selectedlist tells us what we have highlighted - this needs
#	to be cleared at the the next click.
	set selectedlist $listtwo
	set sizeavg [$listtwo size]
	for {set i 0} { $i < $sizeavg } { incr i } {

		set line [$listtwo get $i]
#		the only way that I could get this to work....
		set command "regexp {^$res\ +$num\ +$atom\ +} \$line"
		set ret [eval $command]
		if {$ret == 1} {
			$listtwo see $i
			$listtwo itemconfigure $i -bg grey
			$listtwo itemconfigure $i -fg black
			lappend selectedlist $i
		}
	}

}



#........................................................................
# [Leigh]This is a subroutine re-written by Leigh.  It does the same
# thing as the original subroutine, only significantly faster
#
# For example, on a sequence of 112 residues, with 1799 atoms in total,
# and using 7 .xpk files, the entire program (not just this section)
# has been timed at (on my watch):
# OLD: 50 seconds
# NEW: 4 seconds
#
# Note that the results are saved in the same array as for the
# original subroutine, that is, in an array with the format:
#     array chemicalshift(i) { [PKLIST.PKNUM CHEMSHIFT OLFLAG]* } 
#     where i=the atom number in the sequence
#     an example is  chemicalshift(2) = { hsqc.2 118.5 0 tocsy.5 118.25 0}
#

proc extract_chemicalshifts { allpeaklist } {
global chemicalshift cfold olflag

# 	foreach .xpk file
#	read the file, put info into chemical shift array
	foreach peaklist $allpeaklist {

#		find out what each of the dimensions is
		set dimlabels [nv_peak label $peaklist]

#		foreach peak/line in the peaklist file
#			foreach dimension, extract the info.
		set numpeaks [nv_peak n $peaklist]
		foreachpeak i $peaklist {
# i = the pknum ie. 0 1 2 3 4
			foreach dim $dimlabels {

				set atomlabels [nv_peak elem $dim.L $peaklist.$i]
# ie. peaklist = tocsyhsqc dim = H1
# ie. atom = 11.HN 11.HN (overlap)
				set numlabels [llength $atomlabels]
				if { $numlabels > 1 } {
					set thisol 1
				} else { set thisol 0 }

				for {set j 0} {$j<$numlabels} {incr j 1} { 
					set atomnum [nv_atom num [lindex $atomlabels $j]]
					if {$atomnum == -1} {continue}
					set cs [nv_peak elem $dim.P $peaklist.$i]
					if { $cfold } { 
						set cs [do_c_unfold [lsearch $dimlabels $dim] $peaklist $i $atomnum $cs ] 
					}
					set chemicalshift($atomnum) [format "%s %-.30s.%-4d %s %s " $chemicalshift($atomnum) $peaklist $i $cs $thisol]
#					append chemicalshift($atomnum) "$peaklist.$i $cs $thisol "

	
				}
			}
		}
	}
}



#---------------------------------------------------------------------------
# [Leigh] re-wrote this as part of rewriting the last procedure.
# It is called only if the appropriate checkbox is selected.
# original code from Pascal re-written here.
# this procedure is used if the user thinks that the C peaks
# are folded.  So we attempt to unfold them.
# atom = "111.CA" (for example)

proc do_c_unfold {dindex pkl pknum atomnum cs} {
global amino

	set aname [nv_atom elem aname $atomnum]

# not a C atom - return
	if { ! [regexp {^C} $aname ] } { return $cs; }

	set orig $cs

# sweep width / spectrometer frequency
	set sw [lindex [nv_peak sw $pkl] $dindex]
	set sf [lindex [nv_peak sf $pkl] $dindex]
	set dist [expr $sw/$sf]

# get expected value
	set res [nv_atom elem rname $atomnum]
	set expected $amino($res,$aname,shift)

# which value is it closest to?
	set A [expr abs($expected - ($cs + $dist))] 
	set B [expr abs($expected - $cs)] 
	set C [expr abs($expected - ($cs - $dist))] 
        if { ($A < $B) && ($A < $C) } {
		set cs [expr $cs + $dist]
	} elseif { $C < $B } {
		set cs [expr $cs - $dist]
	} 
# (if it's closest to cs then don't change anything )

# tell user if it has changed.
	if { $orig != $cs } { 
		puts "fs3 -> UNFOLDED: $aname $pkl.$pknum  \[original ppm:$orig\] \[unfolded ppm:$cs\]"
	}

	return $cs

}


#---------------------------------------------------------------------------
# [Leigh] sort the "hits" array for one atom, so that it is clearer
# to read on output.
proc sort_hits { hits } {

#	might as well go and put the first element in, then
#	set the counter i accordingly.
	append sorted "[lindex $hits 0] [lindex $hits 1] [lindex $hits 2] "
	set numbersort 3 

	set numberhits [llength $hits]
	for {set i 4} {$i<$numberhits} {incr i 3} {

		set done 0
		for {set j 1} {($j < $numbersort) && ($done == 0)} {incr j 3} {

			if { [lindex $hits $i] < [lindex $sorted $j] } {
				set sorted [linsert $sorted [expr $j-1] \
					[lindex $hits [expr $i-1]] \
					[lindex $hits $i] \
					[lindex $hits [expr $i+1]] ]
				set numbersort [expr $numbersort + 3]
				set done 1
			}
		}

#		we've made it to the end without finding anything bigger
		if { $done == 0 } {
			append sorted " "
			append sorted "[lindex $hits [expr $i-1]] [lindex $hits $i] [lindex $hits [expr $i+1]] "
			set numbersort [expr $numbersort + 3]
		}
	}

	return $sorted

}


#----------------------------------------------------------------------------
# [Leigh] check each atom to see if it has a violation.  
# there are 2 types of violations possible:
#	TYPE 1) deviations from the average value
#	TYPE 2) deviations from the ppm.out value
# Note that the user can turn each of the types of violations on or off.
#
proc check_violations { i hits resname atomname seqnum average current minimum maximum} {
global nitrogentolerance carbontolerance protontolerance
global vb0 vb1 vb2
global viol_list olflag


# i = the index of the atom number for atomname, resname, and seqnum
# hits = array of matching chemical shifts in all of the experiments
# average = the already calculated average
# current = value from ppm.out file.

	
	set type1 $vb1
	set type2 $vb2

# 	get tolerance
	switch [string range [nv_atom elem aname $i] 0 0] {
		N {set tolerance $nitrogentolerance}
		C {set tolerance $carbontolerance}
		H {set tolerance $protontolerance}
		default { puts "fs3 -> ERROR:  can't get tolerance" }
	}


#	quick check to see if there is a violation, by checking
#	the end points.  If there is no violation, then don't
#	bother with the rest of this procedure. 
#	oob = out of bounds
	set oob1 [expr [expr abs ($average - $minimum)] > $tolerance]
	set oob2 [expr [expr abs ($average - $maximum)] > $tolerance]
#	there might not be a ppm value assigned.
	if {$current < -999} { 
		set oob3 0
		set oob4 0
	} else {
		set oob3 [expr [expr abs ($current - $minimum)] > $tolerance]
		set oob4 [expr [expr abs ($current - $maximum)] > $tolerance]
	}

	if { ($oob1 &&  $type1 ) || \
             ($oob2 &&  $type1 ) || \
             ($oob3 &&  $type2 ) || \
             ($oob4 &&  $type2 ) } { ; 
	} else { return }
	

#	sort the "hits" array to make nicer looking output
#	call it "sorted" now.  Use it from here on in.
	set sorted [sort_hits $hits]


	set numberhits [llength $sorted]
	for {set j 1} {$j<$numberhits} {incr j 3} {

#	check to see if this is an overlap peak, and if we want
#	them in our caclulations
		set isol [lindex $sorted [expr $j + 1]]
		if { ( $olflag == 0 ) && ( $isol ) }  { continue }


		set ppmvalue [lindex $sorted $j]
		set peaklistvalue [lindex $sorted [expr $j - 1]]

#		VIOLATION TYPE 1
		set viol1 0
		if { $type1 } {
			set diff [expr ($ppmvalue - $average)]
			if { abs($diff) > $tolerance } {
				set viol1 $diff
			} 
		}
		

#		VIOLATION TYPE 2
		set viol2 0
		if { $type2 } {
			if {$current > -999} { 
				set diff [expr ($ppmvalue - $current)]
				if { abs($diff) > $tolerance } {
					set viol2 $diff
				}
			}
		}
		set thisviol [list $resname $seqnum $atomname $peaklistvalue $ppmvalue $viol1 $viol2 $isol]
		lappend viol_list $thisviol

	}
}
		

#........................................................................
# [Leigh] 
proc sort_viol { column } {
global curcolumn viol_list


	if  { $column == $curcolumn } { return }
	set curcolumn $column

	if { $curcolumn != 1 } {
		set viol_list [lsort \
		-command mysort $viol_list ]
	} else {
		set viol_list [lsort -integer -index $column $viol_list ]
	}

	printviolations 
}

#........................................................................
# [Leigh] routine to sort the violations from highest to lowest,
# taking the absolute value first
proc mysort { a b } {
global curcolumn

	set absa [expr abs([lindex $a $curcolumn])]
	set absb [expr abs([lindex $b $curcolumn])]

	if { $absa < $absb } { 
		return 1
	} elseif { $absa > $absb } {
		return -1
	} else { return 0 }
}

#........................................................................
# routine to print ou the violations into the violations window.
# first, sort the violations array how the user wants to see it.
# note vb0 = "show violations only"
#      vb1 = "viol(avg)
#      vb2 = "viol(ppm)
proc printviolations {} {
global viol_list
global vb0 vb1 vb2



	.report.cpd24.01 delete 0 end
	foreach violation $viol_list {


# [Leigh wrote everything else in this subroutine]
#	there is probably a better way to do this than foreach...
		foreach "resname seqnum atomname peaklistvalue ppmvalue viol1 viol2 isol" $violation {}

# lots of choices here...
# if you want to print out everything, 
# OR you want vb1 violations and there is one,
# OR you want vb2 violations and there is one...
		if { (! $vb0) ||  ($vb1 && $viol1) || ($vb2 && $viol2) } {

			set line [format "%-3s %4s %4s %-35s %7.3f" \
			$resname $seqnum $atomname $peaklistvalue $ppmvalue ]

			if { $vb1 && ! $viol1 }  {
				set line [format "%s        " $line ]
			} elseif { $vb1 } {
				set line [format "%s %7.3f" $line $viol1 ]
			}

			if { $vb2 && ! $viol2 } {
				set line [format "%s        " $line ]
			} elseif { $vb2 } {
				set line [format "%s %7.3f" $line $viol2 ]
			}

			if {$isol} { set line [format "%s *" $line] }

		.report.cpd24.01 insert end $line
		}

	}
}


	



#........................................................................
proc fs3 {} {
global VERSION

global Nv_Natoms
global previndex1 previndex2
global curcolumn

	set curcolumn 1
	set previndex1 -1
	set previndex2 -1

puts "fs3 -> fs3 version $VERSION has started up"
# first check if there is a sequence loaded in memory
if {$Nv_Natoms==0} {
	bell
	puts "fs3 -> Prior to using this program you need to have a sequence loaded"
	return
	}

if {![winfo exists .fs3]} {
       makefillshiftsavg
    } else {
     wm deiconify .fs3
    }
}



#----------------------------------------------------------------------
# Procedure to put up the GUI
# [leigh] re-wrote most of this.
proc makefillshiftsavg {} {
global nitrogentolerance carbontolerance protontolerance VERSION

	set f .fs3
	destroy $f
	toplevel $f
	wm title $f "FS3 - (Willard & Gagne)"

# program name & comments  comments
	pack [frame $f.f6 ]
   	pack [frame $f.f6.text]
   	pack [frame $f.f6.det -relief sunken -bd 3 ]
	pack [label $f.f6.text.lab1  -foreground darkblue \
		-text " 
FS3 (fillshifts) v. $VERSION "\
		 -font {Helvetica -14 bold} ]

	pack [label $f.f6.text.lab3  -font {Helvetica -12 bold } \
		-foreground darkblue -text "A program to average chemical shifts and find violations"]

	pack [label $f.f6.text.lab2 -foreground darkblue \
-text {Authors:  v 3.X Leigh Willard & Stephane Gagne (Laval University) 
v 2.X Pascal Mercier & Brian Sykes (University of Alberta)} \
	-font {Helvetica -10} ]


# frame containing 3 frames (top of window)
	pack [frame $f.f1] -expand true -fill x

# frame for available peak lists

	pack [frame $f.f1.apl ] -pady 20 -side left -expand true -fill x
	pack [label $f.f1.apl.lab -text "Available peak lists:" \
		-font {Helvetica -12 bold} ]
	pack [frame $f.f1.apl.fr \
        	-borderwidth 1  -relief raised ] -expand true -fill x
	pack [scrollbar $f.f1.apl.fr.sb1 -background grey \
        	-width 10 -borderwidth 1 -command {.fs3.f1.apl.fr.lb xview} \
		-orient horizontal ] -side bottom -fill x
	pack [listbox $f.f1.apl.fr.lb \
		-selectmode extended \
        	-font {Helvetica -12} -width 15 -height 7 \
	        -xscrollcommand {.fs3.f1.apl.fr.sb1 set} \
       	 	-yscrollcommand {.fs3.f1.apl.fr.sb2 set} ] -side left -expand true -fill x
	pack [scrollbar $f.f1.apl.fr.sb2 -background grey \
        	-width 10 -borderwidth 1 -command {.fs3.f1.apl.fr.lb yview}  \
		 ] -side left -fill y

	bind .fs3.f1.apl.fr.lb  <Double-1> { AddSelection }
	proc AddSelection {} {
#		check to see if anything is selected.
		if {! [llength [.fs3.f1.apl.fr.lb curselection]] } {return }

		if {[selection own]==".fs3.f1.apl.fr.lb"} {
		set mylist [split [selection get]]
		foreach i $mylist { .fs3.f1.plu.fr.lb insert end $i }
   		foreach ix [lsort -dec [.fs3.f1.apl.fr.lb curselection] ] {
			.fs3.f1.apl.fr.lb delete $ix
		}
	} }

# frame for buttons 
	pack [frame $f.f1.f2 ] -side left -expand true -fill x 
	pack [button $f.f1.f2.but1 \
		-activebackground darkblue \
		-padx 11 -pady 4 -width 8 -text "Add->" \
		-font {Helvetica -12} -command {AddSelection}] 
	pack [button $f.f1.f2.but2 \
		-activebackground darkblue \
		-padx 11 -pady 4 -width 8 -text "<-Remove" \
		-font {Helvetica -12} -command {DeleteSelection} ] 
	pack [label $f.f1.f2.lab -text "you may use
<shift> + button
<ctrl> + button
to select 
multiple fields
" \
		-foreground red -font {Helvetica -10} ]

# frame for peak lists used 	
	pack [frame $f.f1.plu] -side left -expand true -fill x 
	pack [label $f.f1.plu.lab -borderwidth 1  -text {Peak lists to use:} \
		 -font {Helvetica -12 bold} ] 
	pack [frame $f.f1.plu.fr  \
		 -borderwidth 1  -height 30 -relief raised -width 30 ] \
		-expand true -fill x
    	pack [scrollbar $f.f1.plu.fr.sb1 -borderwidth 1 -background grey \
		-command {.fs3.f1.plu.fr.lb  xview} \
		-orient horizontal -width 10 ] -side bottom -fill x
    	pack [listbox $f.f1.plu.fr.lb \
		-selectmode extended \
        	-font {Helvetica -12} -width 15 -height 7 \
        	-xscrollcommand {.fs3.f1.plu.fr.sb1 set} \
        	-yscrollcommand {.fs3.f1.plu.fr.sb2 set} ] -side left -expand true -fill x
    	pack [scrollbar $f.f1.plu.fr.sb2 -borderwidth 1 -background grey \
		-command {.fs3.f1.plu.fr.lb yview} -width 10 ] \
		-side right -fill y
	bind .fs3.f1.plu.fr.lb <Double-1> { DeleteSelection }
	proc DeleteSelection {} {

#		check to see if anything is selected.
		if {! [llength [.fs3.f1.plu.fr.lb curselection]] } {return }

		if {[selection own]==".fs3.f1.plu.fr.lb"} {
		set mylist [split [selection get]]
		foreach i $mylist { .fs3.f1.apl.fr.lb insert end $i }
   		foreach ix [lsort -dec [.fs3.f1.plu.fr.lb curselection]] {
   			.fs3.f1.plu.fr.lb delete $ix
		}
	} }

# insert the list of available peak lists in the first list
	.fs3.f1.apl.fr.lb delete 0 end
	.fs3.f1.plu.fr.lb delete 0 end
	foreach list [nv_peak list] { .fs3.f1.apl.fr.lb insert end $list}

# frame for tolerances 
	#[Leigh] set default tolerance value from smg
	set protontolerance 0.02
	set nitrogentolerance 0.1
	set carbontolerance 0.1

	pack [frame $f.f2 ] -fill x
    	pack [label $f.f2.lab \
        	-borderwidth 1  -text {Tolerance settings:} \
		-font {Helvetica -12 bold} ]
	pack [frame $f.f2.fH]
	pack [label $f.f2.fH.labH \
	       	-anchor w  -text H -font {Helvetica -12}] -side left
	pack [entry $f.f2.fH.entH \
         	-highlightthickness 0 -font {Helvetica -12} \
		-textvariable protontolerance -width 5 ] -side left
	pack [label $f.f2.fH.typ \
		-font {Helvetica -10} \
		-foreground red -text {(typical values .02 - .03)} ]
	pack [frame $f.f2.fN]
	pack [label $f.f2.fN.labN \
        	-anchor w  -text N -font {Helvetica -12}]  -side left
	pack [entry $f.f2.fN.entN \
         	-highlightthickness 0 -font {Helvetica -12} \
		-textvariable nitrogentolerance -width 5 ] -side left
	pack [label $f.f2.fN.typ \
		-font {Helvetica -10} \
		-foreground red -text {(typical values .10 - .30)} ]
	pack [frame $f.f2.fC] 
	pack [label $f.f2.fC.labC \
        	-anchor w  -text C -font {Helvetica -12}] -side left
	pack [entry $f.f2.fC.entC \
         	-highlightthickness 0 -font {Helvetica -12} \
		-textvariable carbontolerance -width 5 ] -side left
	pack [label $f.f2.fC.typ \
		-font {Helvetica -10} \
		-foreground red -text {(typical values .10 - .30)} ]


## [Leigh] Violation Output Options 
   	pack [frame $f.f3 ] -fill x
   	pack [frame $f.f3.box ]
   	pack [label $f.f3.box.lbl -text {Violation Output:} \
		-font {Helvetica -12 bold } ]
   	pack [checkbutton $f.f3.box.vb0 -text {show violations only} \
		-selectcolor darkblue \
		-font {Helvetica -12} ] -anchor w
   	pack [checkbutton $f.f3.box.vb1 -text {viol(avg)} \
		-selectcolor darkblue \
		-font {Helvetica -12}  ] -anchor w
   	pack [checkbutton $f.f3.box.vb2 -text {viol(ppm)}  \
		-selectcolor darkblue \
		-font {Helvetica -12} ] -anchor w
   	$f.f3.box.vb0 deselect
   	$f.f3.box.vb1 select 
   	$f.f3.box.vb2 select 


## [Leigh] Options 
	pack [frame $f.f4 ] -fill x
   	pack [label $f.f4.lbl -text {Options:} \
		-font {Helvetica -12 bold } ]
	pack [checkbutton $f.f4.bt1 \
		-selectcolor darkblue \
		-text {automatic C peak unfolding} -variable cfold ]
	pack [checkbutton $f.f4.bt2 \
		-selectcolor darkblue \
		-text "include overlapping peaks" -variable olflag ]
   	$f.f4.bt1 deselect
   	$f.f4.bt2 deselect


## bottom buttons
	pack [frame $f.f5 ] -fill both -expand true
	pack [frame $f.f5.box] -pady 20
	pack [button $f.f5.box.but1 \
        	-activebackground darkblue  -width 10 \
		-font {Helvetica -12 bold} -padx 11 -pady 4 -text Proceed \
		-command {doproceed}] -side left
	pack [button $f.f5.box.but2 \
        	-activebackground red  -width 10 \
		-font {Helvetica -12 bold} -padx 11 -pady 4 -text Quit \
		-command {destroy .fs3}] -side left
	pack [button $f.f5.box.but3 \
        	-activebackground darkblue  -width 10 \
		-font {Helvetica -12 bold} -padx 2 -pady 4 -text Help \
		-command {write_main_help} ] -side left
	pack [button $f.f5.box.but4 \
        	-activebackground darkblue  -width 10 \
		-font {Helvetica -12 bold} -padx 2 -pady 4 -text Credits \
		-command {give_smg_details} ] -side left

}



########################################
#
# This is where the fun begins
#
########################################

proc doproceed {} {

global nitrogentolerance carbontolerance protontolerance curcolumn

	set curcolumn 1

# check if the user has chosen at least one peak list to work with
if {[.fs3.f1.plu.fr.lb size]==0} {
	bell
	puts "fs3 -> ** please select at least one peak list"
return
}

# check if the user has tolerance values for H, N and C
set protontolerance    [.fs3.f2.fH.entH get]
set nitrogrentolerance [.fs3.f2.fN.entN get]
set carbontolerance    [.fs3.f2.fC.entC get]

if {$protontolerance<0 || $nitrogrentolerance<0 || $carbontolerance<0} {
	bell
	puts "fs3 -> ** For the program to work you need to set tolerance values for proton, nitrogen and carbon atoms.  One of them is not set, or one of them is set to a negative value"
	return
}

###########################################
# core of the procedure
###########################################

global olflag
global amino
global chemicalshift
global Nv_Natoms
global viol_list
global selectedlist
set selectedlist {}
set viol_list ""
set numberatoms $Nv_Natoms

# store all the peak lists chosen in a variable called allpeaklist
set allpeaklist [.fs3.f1.plu.fr.lb get 0 end]

puts "fs3 -> getting mean chemical shifts from BMRB"
  if {![info exists amino]} then {
	# the GetAminoTables is from the Hasting dribution of nmrview (script resdata.tcl)
        GetAminoTables
    }

puts "fs3 -> starting analysis"


#  put up the report window
   makereportfillshifts


#.......................................
# [Leigh] section that Leigh re-wrote
# initialize chemicalshifts 
for {set i 0} {$i<=$numberatoms} {incr i 1} {
	set chemicalshift($i) ""
}
extract_chemicalshifts $allpeaklist
# end section
#.......................................


# [Leigh] re-wrote most of the following code.
# before, if there was a ppm.out value, but no corresponding .xpk
# values, then this section was skipped.  Now we do this anyway.
# This meant re-writing almost all of the following section.

# Average the shifts and report possible assignment errors


for {set i 1} {$i<=$numberatoms} {incr i 1} { 

  set foundhits 0
  set hits $chemicalshift($i)
  set numberhits [llength $hits] 

  set resname         [nv_atom elem rname $i]
  set atomname        [nv_atom elem aname $i]
  set seqnum          [nv_atom elem seq $i]

# check to see if this is a bogus amino acid.  Not everything in
# the nv_atom array is a valid atom.  For example, MET,HT1 is listed
# in the nv_atom structure, but this combination is bogus
  if ![info exists amino($resname,$atomname,shift)] {continue}

  # get total chemical shifts, and counts.
  set total 0
  set minimum 999
  set maximum -999
  for {set j 1} {$j<$numberhits} {incr j 3} {


#	check to see if this is an overlap peak, and if we want
#	them in our caclulations
	if { ( $olflag == 0 ) && ( [lindex $hits [expr $j + 1]] ) } {
# debug
# puts "fs3 -> overlap at atom# $i peak $hits at index [expr $j + 1] "
		continue
	}

	incr foundhits 1
	set ppmvalue [lindex $hits $j]
	set total [expr $total + $ppmvalue]
	set minimum [expr {($ppmvalue < $minimum) ? $ppmvalue:$minimum}]
	set maximum [expr {($ppmvalue > $maximum) ? $ppmvalue:$maximum}]
  }


  set currentppm [nv_atom elem ppm $i]
  set BMRB $amino($resname,$atomname,shift)
  set deltaBMRB_S "ND"
  set delta_S "ND"
  set currentppm_S "ND"
  set average_S "ND"

# only if there were matching atom values in the .xpk files
  if {$foundhits > 0} {

	set average [expr $total/$foundhits]
#       [Leigh] set average off to 3 decimal places. 
#	otherwise rounding errors show up later...
	set average [expr round ($average * 1000) / 1000.0 ]
        set average_S [format "%7.3f" $average]

  	set deltaBMRB [expr $average - $BMRB]
	set deltaBMRB_S [format "%7.3f" $deltaBMRB]


  	check_violations $i $hits $resname $atomname $seqnum \
		$average $currentppm $minimum $maximum

  	if {$currentppm>-1000} {
		set delta [expr $average-$currentppm]
 		set delta_S [format "%7.3f" $delta]
  	}
  }
  if {$currentppm>-1000} { set currentppm_S [format "%7.3f" $currentppm] }

  
  .report.cpd28.01 insert end \
	[format "%-3s %4s %4s    %7s %11s%11s%11.3f%11s" \
	$resname $seqnum $atomname $average_S $currentppm_S \
	$delta_S $BMRB $deltaBMRB_S]

}

printviolations
puts "fs3 -> done analysis"
}

# end doproceed




###########################
#
# Assign Shift procedure
#
###########################

proc AssignShift {} {
if {[selection own]==".report.cpd28.01"} {
	set selected [selection get]
	set residuenum [lindex $selected 1]
	set atomname [lindex $selected 2]
	set avg [lindex $selected 3]
	if {$avg != "ND"} {
		set atomnumber [nv_atom num $residuenum.$atomname]
		puts "fs3 -> assign $residuenum.$atomname $avg"
		nv_atom elem ppm $atomnumber $avg
	} else {
		puts "fs3 -> !! cannot assign $residuenum.$atomname"
	}
} else { puts "fs3 -> ** you must click on a residue to Assign" }
}


###########################
#
# Assign All Shift procedure
#
###########################

proc AssignAllShift {} {
global Nv_Prefs

# [Leigh] Stephane would like the program to make a copy of
# the ppm.out file before doing an assign all.  This is how
# we do it:
set origppm $Nv_Prefs(file,ppm)
set mytime [clock format [clock seconds] -format "%d%b%y_%H:%M"]
set newppm "${origppm}_${mytime}"
# a nmrview routine
writeppm $newppm
puts "fs3 -> Although \"Assign All\" does not modify your ppm.out file,"
puts "fs3 -> it does make a copy of your current values before assigning."
puts "fs3 -> these can be found in $newppm "
 
for {set i 0} {$i<[.report.cpd28.01 size]} {incr i} {
	set currentline [.report.cpd28.01 get $i $i]
	set selected [lindex $currentline 0]
	set residuenum [lindex $selected 1]
	set atomname [lindex $selected 2]
	set avg [lindex $selected 3]
	if {$avg == "ND"} { continue }
	set atomnumber [nv_atom num $residuenum.$atomname]
#	puts "fs3 -> assign $residuenum.$atomname $avg"
	nv_atom elem ppm $atomnumber $avg
}
puts "fs3 -> all ppm values were assigned."
}

###########################
#
# Save Shift report
#
###########################


proc SaveShiftReport {} {
global tk_version tk_strictMotif
global viol_title

set tk_strictMotif 0

set file [tk_getSaveFile -initialdir [pwd]  -initialfile "fs3.report"]

# [Leigh] changed to catch a cancelled operation
if [catch {open $file w} createreportfile] {return;}


puts $createreportfile "#\n#\n# Automatically generated output from FS3"
puts $createreportfile "#\n#\n#\n"
puts $createreportfile "# Peak lists used:"
puts $createreportfile [.fs3.f1.plu.fr.lb get 0 end]

puts $createreportfile "\n\n# Chemical Shift Violations:"
puts $createreportfile $viol_title 

#write errors at the top
for {set i 0} {$i<[.report.cpd24.01 size]} {incr i} {
	set currentline [.report.cpd24.01 get $i $i]
	set currentline [lindex $currentline 0]
	puts $createreportfile $currentline
}

puts $createreportfile "\n\n\n"

puts $createreportfile [format "%-3s %4s %4s    %7s %11s%11s%11s%11s" RES NUM ATOM ppm.out average delta BMRB deltaBMRB]
for {set i 0} {$i<[.report.cpd28.01 size]} {incr i} {
	set currentline [.report.cpd28.01 get $i $i]
	set currentline [lindex $currentline 0]
	puts $createreportfile $currentline
}


close $createreportfile
}

#..........................................................................
# 
proc makereportfillshifts {} {
global viol_title
set base .report

    destroy $base
    toplevel $base
    wm title $base "FS3 Report"

#
#   header lines
    pack [label $base.lab1  -borderwidth 1  \
        -text {Chemical Shift Violations} \
	-font {Helvetica -14 bold } -foreground darkblue ] -pady 10
    pack [label $base.lab2 \
	-text {click on a button to sort on that column
click on a line to highlight the corresponding average result
* at the end of a line indicates an overlapped peak} \
	-font {Helvetica -12 } -foreground red ]  -pady 10

#
#   print out the title lines
    set_viol_title $base

#
#   set up the first listbox
    pack [frame $base.cpd24 \
        -borderwidth 1  -relief raised ] -anchor w
    pack [listbox $base.cpd24.01 -height 10 -width 75 \
        -font {courier -12} \
        -yscrollcommand {.report.cpd24.03 set} ] -fill x -side left
    pack [scrollbar $base.cpd24.03 \
        -borderwidth 1 -command {.report.cpd24.01 yview} -width 10 ] \
	-side right -fill y
    bind $base.cpd24.01 <ButtonRelease> {select_violation ".report.cpd24.01" ".report.cpd28.01"}


#
# second title & label 
    pack [label $base.lab3 \
        -borderwidth 1 -text {Chemical shift averaging results} \
	-foreground darkblue -font {Helvetica -14 bold } ] -pady 10
    pack [label $base.lab4 \
	-text {click on a line to highlight any corresponding violations} \
	-font {Helvetica -12} -foreground red ] -pady 8
    pack [label $base.lab34 \
        -borderwidth 1  -relief flat  -text [format " %-3s %4s %4s    %7s %11s%11s%11s%11s" RES  NUM ATOM AVERAGE PPM.OUT deltaPPM BMRB deltaBMRB] -font {courier -12} -justify left ] -anchor w

#
# chemical shifts averaging table
    pack [frame $base.cpd28 \
        -borderwidth 1  -relief raised ] -anchor w
    pack [listbox $base.cpd28.01 -height 10 -width 75 \
         -font {courier -12}\
         -yscrollcommand {.report.cpd28.03 set}  ] -side left
    pack [scrollbar $base.cpd28.03 \
        -borderwidth 1 -command {.report.cpd28.01 yview} -width 10 ] \
	-side right -fill y

    bind $base.cpd28.01 <ButtonRelease> { select_violation ".report.cpd28.01" ".report.cpd24.01" }

#
# bottom buttons
    set btns [frame $base.buts]
    pack [button $btns.but1 \
         -padx 11 -pady 4 -text Assign -font {Helvetica -12 bold} -activebackground darkblue -command {AssignShift} ] -side left
    pack [button $btns.but2 \
         -padx 11 -pady 4 -text {Assign All} -font {Helvetica -12 bold} -activebackground darkblue -command {AssignAllShift} ] -side left
    pack [button $btns.but3 \
         -padx 11 -pady 4 -text {Save report} -font {Helvetica -12 bold} -activebackground darkblue  -command {SaveShiftReport} ] -side left
    pack [button $btns.but4 \
         -padx 11 -pady 4 -text Done -font {Helvetica -12 bold} -activebackground red -command {destroy .report} ] -side left
    pack [button $btns.but5 \
         -padx 11 -pady 4 -text Help -font {Helvetica -12 bold} -activebackground darkblue -command {write_report_help}  ] -side right
   pack $btns -side bottom -fill x -padx 10 -pady 20

}
